# Copyright (c) 2014-2017 Pivotal Software, Inc. All rights reserved.
#
# Unauthorized use, copying or distribution of this source code via any
# medium is strictly prohibited without the express written consent of
# Pivotal Software, Inc.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

require 'uaa'
require 'net/ssh'
require_relative 'ops_man_info'

module PcfPause
  class Pause
    attr_reader :logger

    def initialize(logger = Logger.new(STDOUT))
      @logger = logger
    end

    TARGET_JOBS = %w(
      nats consul_server mysql_proxy etcd_server etcd_tls_server diego_database nfs_server router ccdb
      uaadb consoledb cloud_controller ha_proxy health_manager clock_global
      cloud_controller_worker collector uaa diego_brain diego_cell doppler
      loggregator_trafficcontroller tcp_router
    )

    def get_job_sequence(bosh_vms_value)
      bosh_vms_value.select do |vm_name|
        TARGET_JOBS.any? { |job| vm_name.start_with?(job) }
      end
    end

    def smoke_tests(url, username, password, key_path='')
      bosh_command, bosh_vms_value = connect_to_bosh(password, url, username)

      Net::SSH.start(URI.parse(url).host, 'ubuntu', keys: [key_path]) do |ssh|
        exec_ssh ssh, "#{bosh_command} #{smoke_tests_command}"
      end
      logger.info 'Smoke tests have finished'
    end

    def start(url, username, password, key_path='')
      bosh_command, bosh_vms_value = connect_to_bosh(password, url, username)

      logger.info("Preparing to start instances")

      Net::SSH.start(URI.parse(url).host, 'ubuntu', keys: [key_path]) do |ssh|
        exec_ssh ssh, "#{bosh_command} #{start_command}"
        logger.info "Started"
      end
      logger.info 'All instances started'
    end

    def stop(url, username, password, key_path='')
      bosh_command, bosh_vms_value = connect_to_bosh(password, url, username)

      stop_sequence = get_job_sequence(bosh_vms_value).reverse

      logger.info("Preparing to stop instances #{stop_sequence.join(', ')}")
      Net::SSH.start(URI.parse(url).host, 'ubuntu', keys: [key_path]) do |ssh|
        stop_sequence.each do |job_name|
          exec_ssh ssh, "#{bosh_command} #{stop_command(job_name)}"
          logger.info "Stopped #{job_name}"
        end
      end
      logger.info 'All instances stopped'
    end

    def connect_to_bosh(password, url, username)
      logger.info("Connecting to OpsMan at #{url}")
      ops_man = PcfPause::OpsManInfo.new(url, username, password)

      manifest = ops_man.get_director_manifest
      uaa_secret = manifest['jobs'][0]['properties']['uaa']['clients']['ops_manager']['secret']
      director_ip = manifest['jobs'][0]['properties']['director']['address']

      product_id = ops_man.get_product_id 'cf'
      bosh_vms_value = ops_man.get_vm_names product_id
      bosh_command = "BOSH_CLIENT=ops_manager BOSH_CLIENT_SECRET=#{uaa_secret} BUNDLE_GEMFILE=/home/tempest-web/tempest/web/vendor/bosh/Gemfile bundle exec bosh -n --ca-cert /var/tempest/workspaces/default/root_ca_certificate -d /var/tempest/workspaces/default/deployments/#{product_id}.yml -t #{director_ip}"
      return bosh_command, bosh_vms_value
    end

    def start_command
      "start --force"
    end

    def stop_command(job_name)
      "stop #{job_name} --hard"
    end

    def smoke_tests_command
      "run errand smoke-tests"
    end

    def exec_ssh(ssh, command)
      logger.debug 'SSH: executing command'
      ssh.exec!(command) do |ch, stream, data|
        puts data
        ch.on_request "exit-status" do |ch, data|
          exit_status = data.read_long
          raise "process terminated with exit status: #{exit_status}" if exit_status != 0
        end
      end
    end
  end
end
